/*
 * \brief  Azure SDK error checking helper functions
 * \author Timo Nicolai
 * \date   2023-02-23
 */

/*
 * Copyright (C) 2023 gapfruit AG
 */

#pragma once

/* Genode includes */
#include <base/exception.h>
#include <base/log.h>

/* Azure includes */
extern "C" {

#include <azure/az_iot.h>

}


namespace Azure {

	struct Exception : Genode::Exception { };

	void check(::az_result rc, char const *what)
	{
		if (!::az_result_failed(rc))
			return;

		using Genode::Hex;

		Genode::error("Azure: failed to ", what, ": ",
		              "error code ", Hex(rc, Hex::PREFIX, Hex::PAD));

		throw Exception { };
	}

	inline bool ok(::az_iot_status status)
	{
		if (::az_iot_status_succeeded(status))
			return true;

		Genode::error("Azure: invalid IoT status code: ",
		              static_cast<int>(status));

		return false;
	}
}
