/*
 * \brief  MQTT error checking helper functions
 * \author Timo Nicolai
 * \date   2023-02-25
 */

/*
 * Copyright (C) 2023 gapfruit AG
 */

#pragma once

/* Genode includes */
#include <base/exception.h>
#include <base/log.h>
#include <util/string.h>

/* Mosquitto includes */
extern "C" {

#include <mosquitto.h>

}

/* libc includes */
extern "C" {

#include <errno.h>
#include <string.h>

}


namespace MQTT {

	struct Exception : Genode::Exception
	{
		using What = Genode::String<256>;

		int  rc;
		What what;

		explicit Exception(int rc, What const &what)
		:
			rc { rc },
			what { what }
		{ }
	};

	inline void check(int rc, char const *action)
	{
		if (rc == MOSQ_ERR_SUCCESS)
			return;

		Exception::What what;

		if (rc == MOSQ_ERR_ERRNO) {
			char buf [1024];
			if (::strerror_r(errno, buf, sizeof(buf)) == 0)
				what = Exception::What { "failed to ", action, ": ", Genode::Cstring { buf } };
			else
				what = Exception::What { "failed to ", action };
		} else {
			what = Exception::What { "failed to ", action, ": ", ::mosquitto_strerror(rc) };
		}

		Genode::error(what);

		throw Exception { rc, what };
	}
}
