/*
 * \brief  Obtain formatted timestamps
 * \author Timo Nicolai
 * \date   2022-11-07
 */

/*
 * Copyright (C) 2022 gapfruit AG
 */

#pragma once

/* Genode includes */
#include <base/log.h>
#include <libc/component.h>
#include <util/string.h>

/* libc includes */
extern "C" {

#include <stddef.h>
#include <string.h>
#include <time.h>

}


namespace Util {

	enum : Genode::size_t { MAX_TIMESTAMP_LEN = 18 };

	using Timestamp = Genode::String<MAX_TIMESTAMP_LEN + 1>;

	inline Timestamp timestamp(char const *fmt)
	{
		char buf[MAX_TIMESTAMP_LEN + 1];

		Libc::with_libc([fmt, &buf] {
			::time_t t;
			::tm time;

			if (::time(&t) != -1 && ::gmtime_r(&t, &time)) {
				::strftime(buf, sizeof(buf), fmt, &time);
			} else {
				Genode::error("failed to produce timestamp");
				::strncpy(buf, "unknown_time", sizeof(buf));
			}
		});

		return Timestamp { buf };
	}
}
